# coding=utf-8
# Copyright 2019 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for rockyou dataset module."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from tensorflow_datasets import testing
from tensorflow_datasets.structured import rock_you


class RockYouTest(testing.DatasetBuilderTestCase):
  DATASET_CLASS = rock_you.RockYou

  SPLITS = {
      "train": 11,
  }

  DL_EXTRACT_RESULT = "rockyou.txt"


if __name__ == "__main__":
  testing.test_main()



# coding=utf-8
# Copyright 2019 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding=utf-8
"""Tests for tensorflow_datasets.core.features.text_feature."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import tensorflow as tf
from tensorflow_datasets import testing
from tensorflow_datasets.core import features
from tensorflow_datasets.core.features.text import text_encoder

tf.compat.v1.enable_eager_execution()


class TextFeatureTest(testing.FeatureExpectationsTestCase):

  def test_text(self):
    nonunicode_text = 'hello world'
    unicode_text = u'你好'

    self.assertFeature(
        feature=features.Text(),
        shape=(),
        dtype=tf.string,
        tests=[
            # Non-unicode
            testing.FeatureExpectationItem(
                value=nonunicode_text,
                expected=tf.compat.as_bytes(nonunicode_text),
            ),
            # Unicode
            testing.FeatureExpectationItem(
                value=unicode_text,
                expected=tf.compat.as_bytes(unicode_text),
            ),
            # Empty string
            testing.FeatureExpectationItem(
                value='',
                expected=tf.compat.as_bytes(''),
            ),
        ],
    )

  def test_text_encoded(self):
    unicode_text = u'你好'

    # Unicode integer-encoded by byte
    self.assertFeature(
        feature=features.Text(encoder=text_encoder.ByteTextEncoder()),
        shape=(None,),
        dtype=tf.int64,
        tests=[
            testing.FeatureExpectationItem(
                value=unicode_text,
                expected=[i + 1 for i in [228, 189, 160, 229, 165, 189]],
            ),
            # Empty string
            testing.FeatureExpectationItem(
                value='',
                expected=[],
            ),
        ],
    )

  def test_text_conversion(self):
    text_f = features.Text(encoder=text_encoder.ByteTextEncoder())
    text = u'你好'
    self.assertEqual(text, text_f.ints2str(text_f.str2ints(text)))

  def test_save_load_metadata(self):
    text_f = features.Text(
        encoder=text_encoder.ByteTextEncoder(additional_tokens=['HI']))
    text = u'HI 你好'
    ids = text_f.str2ints(text)
    self.assertEqual(1, ids[0])

    with testing.tmp_dir(self.get_temp_dir()) as data_dir:
      feature_name = 'dummy'
      text_f.save_metadata(data_dir, feature_name)

      new_f = features.Text()
      new_f.load_metadata(data_dir, feature_name)
      self.assertEqual(ids, text_f.str2ints(text))


if __name__ == '__main__':
  testing.test_main()



# coding=utf-8
# Copyright 2019 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for colorectal_histology dataset module."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from tensorflow_datasets import testing
from tensorflow_datasets.image import colorectal_histology

# testing/colorectal_histology.py generates fake input data

num_classes = len(colorectal_histology._CLASS_NAMES)  # pylint: disable=protected-access


class ColorectalHistologyTest(testing.DatasetBuilderTestCase):
  DATASET_CLASS = colorectal_histology.ColorectalHistology
  SPLITS = {
      "train": 2 * num_classes,
  }


class ColorectalHistologyS3Test(ColorectalHistologyTest):
  VERSION = "experimental_latest"


class ColorectalHistologyLargeTest(testing.DatasetBuilderTestCase):
  DATASET_CLASS = colorectal_histology.ColorectalHistologyLarge
  SPLITS = {
      "test": 1,
  }


class ColorectalHistologyLargeS3Test(ColorectalHistologyLargeTest):
  VERSION = "experimental_latest"


if __name__ == "__main__":
  testing.test_main()



# coding=utf-8
# Copyright 2019 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The Multi-Genre NLI Corpus."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os

import tensorflow as tf
from tensorflow_datasets.core import api_utils
import tensorflow_datasets.public_api as tfds

_CITATION = """\
@InProceedings{N18-1101,
  author = "Williams, Adina
            and Nangia, Nikita
            and Bowman, Samuel",
  title = "A Broad-Coverage Challenge Corpus for
           Sentence Understanding through Inference",
  booktitle = "Proceedings of the 2018 Conference of
               the North American Chapter of the
               Association for Computational Linguistics:
               Human Language Technologies, Volume 1 (Long
               Papers)",
  year = "2018",
  publisher = "Association for Computational Linguistics",
  pages = "1112--1122",
  location = "New Orleans, Louisiana",
  url = "http://aclweb.org/anthology/N18-1101"
}
"""

_DESCRIPTION = """\
The Multi-Genre Natural Language Inference (MultiNLI) corpus is a
crowd-sourced collection of 433k sentence pairs annotated with textual
entailment information. The corpus is modeled on the SNLI corpus, but differs in
that covers a range of genres of spoken and written text, and supports a
distinctive cross-genre generalization evaluation. The corpus served as the
basis for the shared task of the RepEval 2017 Workshop at EMNLP in Copenhagen.
"""


class MultiNLIConfig(tfds.core.BuilderConfig):
  """BuilderConfig for MultiNLI."""

  @api_utils.disallow_positional_args
  def __init__(self, text_encoder_config=None, **kwargs):
    """BuilderConfig for MultiNLI.

    Args:
      text_encoder_config: `tfds.features.text.TextEncoderConfig`, configuration
        for the `tfds.features.text.TextEncoder` used for the features feature.
      **kwargs: keyword arguments forwarded to super.
    """
    # Version history:
    # 1.0.0: S3 (new shuffling, sharding and slicing mechanism).
    # 0.0.2: Initial version.
    super(MultiNLIConfig, self).__init__(
        version=tfds.core.Version(
            "0.0.2", experiments={tfds.core.Experiment.S3: False}),
        supported_versions=[
            tfds.core.Version("1.0.0"),
        ], **kwargs)
    self.text_encoder_config = (
        text_encoder_config or tfds.features.text.TextEncoderConfig())


class MultiNLI(tfds.core.GeneratorBasedBuilder):
  """MultiNLI: The Stanford Question Answering Dataset. Version 1.1."""

  BUILDER_CONFIGS = [
      MultiNLIConfig(
          name="plain_text",
          description="Plain text",
      ),
  ]

  def _info(self):
    return tfds.core.DatasetInfo(
        builder=self,
        description=_DESCRIPTION,
        features=tfds.features.FeaturesDict({
            "premise":
                tfds.features.Text(
                    encoder_config=self.builder_config.text_encoder_config),
            "hypothesis":
                tfds.features.Text(
                    encoder_config=self.builder_config.text_encoder_config),
            "label":
                tfds.features.ClassLabel(
                    names=["entailment", "neutral", "contradiction"]),
        }),
        # No default supervised_keys (as we have to pass both premise
        # and hypothesis as input).
        supervised_keys=None,
        urls=["https://www.nyu.edu/projects/bowman/multinli/"],
        citation=_CITATION,
    )

  def _vocab_text_gen(self, filepath):
    for _, ex in self._generate_examples(filepath):
      yield " ".join([ex["premise"], ex["hypothesis"]])

  def _split_generators(self, dl_manager):

    downloaded_dir = dl_manager.download_and_extract(
        "http://storage.googleapis.com/tfds-data/downloads/multi_nli/"
        "multinli_1.0.zip")
    mnli_path = os.path.join(downloaded_dir, "multinli_1.0")
    train_path = os.path.join(mnli_path, "multinli_1.0_train.txt")
    matched_validation_path = os.path.join(mnli_path,
                                           "multinli_1.0_dev_matched.txt")
    mismatched_validation_path = os.path.join(
        mnli_path, "multinli_1.0_dev_mismatched.txt")
    # Generate shared vocabulary
    # maybe_build_from_corpus uses SubwordTextEncoder if that's configured
    self.info.features["premise"].maybe_build_from_corpus(
        self._vocab_text_gen(train_path))
    encoder = self.info.features["premise"].encoder
    # Use maybe_set_encoder because the encoder may have been restored from
    # package data.
    self.info.features["premise"].maybe_set_encoder(encoder)
    self.info.features["hypothesis"].maybe_set_encoder(encoder)

    return [
        tfds.core.SplitGenerator(
            name=tfds.Split.TRAIN,
            num_shards=10,
            gen_kwargs={"filepath": train_path}),
        tfds.core.SplitGenerator(
            name="validation_matched",
            num_shards=1,
            gen_kwargs={"filepath": matched_validation_path}),
        tfds.core.SplitGenerator(
            name="validation_mismatched",
            num_shards=1,
            gen_kwargs={"filepath": mismatched_validation_path}),
    ]

  def _generate_examples(self, filepath):
    """Generate mnli examples.

    Args:
      filepath: a string

    Yields:
      dictionaries containing "premise", "hypothesis" and "label" strings
    """
    for idx, line in enumerate(tf.io.gfile.GFile(filepath, "rb")):
      if idx == 0:
        continue  # skip header
      line = tf.compat.as_text(line.strip())
      split_line = line.split("\t")
      # Examples not marked with a three out of five consensus are marked with
      # "-" and should not be used in standard evaluations.
      if split_line[0] == "-":
        continue
      # Works for both splits even though dev has some extra human labels.
      yield idx, {
          "premise": split_line[5],
          "hypothesis": split_line[6],
          "label": split_line[0]
      }



# coding=utf-8
# Copyright 2019 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for checking that eager isn't enabled by default on importing tfds."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import tensorflow as tf

import tensorflow_datasets as tfds  # pylint: disable=unused-import


class EagerNotEnabledByDefaultTest(tf.test.TestCase):

  def test_eager_is_not_enabled_by_default(self):
    self.assertFalse(tf.executing_eagerly())


if __name__ == '__main__':
  tf.test.main()



# coding=utf-8
# Copyright 2019 The TensorFlow Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Tests for starcraft video dataset."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from tensorflow_datasets import testing
from tensorflow_datasets.video import starcraft


class StarcraftVideoDatasetTest(testing.DatasetBuilderTestCase):
  DATASET_CLASS = starcraft.StarcraftVideo
  BUILDER_CONFIG_NAMES_TO_TEST = ["brawl_64"]

  DL_EXTRACT_RESULT = {
      "valid": "valid.tfrecords",
      "test": "test.tfrecords",
      "train_0": "train_0.tfrecords",
      "train_1": "train_1.tfrecords"
  }

  SPLITS = {
      "train": 2,
      "test": 1,
      "validation": 1,
  }


class StarcraftVideoDataset128Test(testing.DatasetBuilderTestCase):
  """Separate test to cover the 128x128 resolution videos."""
  DATASET_CLASS = starcraft.StarcraftVideo
  BUILDER_CONFIG_NAMES_TO_TEST = ["brawl_128"]

  DL_EXTRACT_RESULT = {
      "valid": "128_valid.tfrecords",
      "test": "128_test.tfrecords",
      "train_0": "128_train_0.tfrecords",
      "train_1": "128_train_1.tfrecords"
  }

  SPLITS = {
      "train": 2,
      "test": 1,
      "validation": 1,
  }


if __name__ == "__main__":
  testing.test_main()
